#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

#include <sys/types.h>
#include <iostream>
#include <memory>

#include "resiprocate/DeprecatedDialog.hxx"
#include "resiprocate/Helper.hxx"
#include "resiprocate/SipMessage.hxx"
#include "resiprocate/SipStack.hxx"
#include "resiprocate/Uri.hxx"
#include "resiprocate/os/Logger.hxx"

using namespace resip;
using namespace std;

#define RESIPROCATE_SUBSYSTEM Subsystem::SIP

int
main(int argc, char* argv[])
{
	Log::initialize(Log::Cout, Log::Err, argv[0]);

	const int			CLIENT_PORT = 6050;
	const int			SERVER_PORT = 5060;

   SipStack sendStack;
   SipStack recvStack;
   sendStack.addTransport(UDP, CLIENT_PORT);
   recvStack.addTransport(UDP, SERVER_PORT);

   NameAddr dest;
   dest.uri().scheme() = "sip";
   dest.uri().user() = "fluffy";
   dest.uri().host() = "127.0.0.1";
   dest.uri().port() = SERVER_PORT;
   dest.uri().param(p_transport) = "udp";
   
   NameAddr from;
   from.uri().scheme() = "sip";
   from.uri().user() = "snoopy";
   from.uri().host() = "127.0.0.1";
   from.uri().port() = CLIENT_PORT;
   from.uri().param(p_transport) = "udp";
   
   int totalCalls = 500;

   bool done[10000];
   assert( sizeof(done)/sizeof(bool) > (unsigned int)(totalCalls+1) );
   for ( int ii=0; ii<=totalCalls; ii++ )
   {
      done[ii]=false;
   }
   
   int lastSent=0;
   int lastRecv=0;
   int i=0;
   
   UInt64 startTime = Timer::getTimeMs();

   while ( lastRecv < totalCalls )
   {
      InfoLog( << "loop " << i++  );
      
      if ( (lastRecv >= lastSent) && (lastSent <= totalCalls) )
      { 
         Data count( ++lastSent );
         from.uri().user() = count;
         auto_ptr<SipMessage> message(Helper::makeInvite( dest, from, from));
         sendStack.send(*message);
         InfoLog( << "sendStack sent msg from user: " << count );
      }
         
      FdSet sendFdSet, recvFdSet;
      sendStack.buildFdSet(sendFdSet);
      recvStack.buildFdSet(recvFdSet);
      int err1 = sendFdSet.selectMilliSeconds(0);
      int err2 = recvFdSet.selectMilliSeconds(0);
      assert (err1 != -1);
      assert (err2 != -1);

      sendStack.process(sendFdSet);
      recvStack.process(recvFdSet);

      SipMessage* received1 = (sendStack.receive());
      if (received1)
      {
         InfoLog (<< "sendStack got: " << received1->brief() 
                  << " from user: " << received1->header(h_From).uri().user() );

         int user = atoi( received1->header(h_From).uri().user().c_str() );
      
         if ( (received1->isResponse()) )
         {
            if ( received1->header(h_StatusLine).responseCode() == 200 )
            {
               if ( !done[user] )
               {
                  DebugLog(<< "Creating dialog.");
                  done[user]=true;
               
                  DeprecatedDialog dlog(from);
               
                  DebugLog(<< "Creating dialog as UAC.");
                  dlog.createDialogAsUAC(*received1);
               
                  DebugLog(<< "making ack.");
                  auto_ptr<SipMessage> ack(dlog.makeAck(*received1) );
               
                  DebugLog(<< "making bye.");
                  auto_ptr<SipMessage> bye(dlog.makeBye());
               
                  DebugLog(<< "Sending ack: << *ack");
                  sendStack.send(*ack);

                  DebugLog(<< "Sending bye: << *bye");
                  sendStack.send(*bye);  
               }
               else
               { 
                  if ( user > lastRecv )
                  {
                     lastRecv = user;
                  }
               }
            }
         }     
         
         delete received1;
      }
               
      SipMessage* received2 = (recvStack.receive());
      if (received2)
      {
         InfoLog (<< "recvStack got: " << received2->brief() 
                  << " from user: " << received2->header(h_From).uri().user() );   
 
         MethodTypes meth = received2->header(h_RequestLine).getMethod();

         if ( meth == INVITE )
         {
            //Data localTag = Helper::computeTag(4);

            auto_ptr<SipMessage> msg180(Helper::makeResponse(*received2, 180, dest, "Ringing"));
            //msg180->header(h_To).uri().param(p_tag) = localTag;
            recvStack.send( *msg180);
            
            auto_ptr<SipMessage> msg200(Helper::makeResponse(*received2, 200, dest, "OK"));
            //msg200->header(h_To).uri().param(p_tag) = localTag;
            recvStack.send(*msg200);
         }

         if ( meth == BYE )
         {
            auto_ptr<SipMessage> msg200(Helper::makeResponse(*received2, 200, dest, "OK"));
            InfoLog (<< "recvStack got bye - send 200 : " << *msg200 );   
            
            recvStack.send(*msg200);
         }

         delete received2;
      }
   }
  
   UInt64 elapsed = Timer::getTimeMs() - startTime;
   cout << totalCalls << " calls peformed in " << elapsed << " ms, a rate of " 
        << totalCalls / ((float) elapsed / 1000.0) << " calls per second.]" << endl;

   return 0;
}
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
